local AceHealVar = {}
AceHealVar.DEFAULT_OPTIONS = {
	threshold = .8,
	cancelthreshold = .5,
	ButtonEvents = {},
	SpellMsg = TRUE,
	ClickRes = TRUE,
}
AceHealVar.AddonList = {}

AceHealVar.ClassHealth = {
   [AceHealLocals.CLASS_DRUID]   = { ["1"] = 30, ["60"] = 3500 },
   [AceHealLocals.CLASS_MAGE]    = { ["1"] = 30, ["60"] = 2500 },
   [AceHealLocals.CLASS_HUNTER]  = { ["1"] = 30, ["60"] = 3500 },
   [AceHealLocals.CLASS_PALADIN] = { ["1"] = 30, ["60"] = 4000 },
   [AceHealLocals.CLASS_PRIEST]  = { ["1"] = 30, ["60"] = 2500 },
   [AceHealLocals.CLASS_ROGUE]   = { ["1"] = 30, ["60"] = 3500 },
   [AceHealLocals.CLASS_SHAMAN]  = { ["1"] = 30, ["60"] = 3800 },
   [AceHealLocals.CLASS_WARLOCK] = { ["1"] = 30, ["60"] = 3500 },
   [AceHealLocals.CLASS_WARRIOR] = { ["1"] = 30, ["60"] = 5000 },
}

--[[--------------------------------------------------------------------------------
  Class Setup
-----------------------------------------------------------------------------------]]
AceHeal = AceAddon:new({
    name          = AceHealLocals.Title,
    description   = AceHealLocals.Desc,
    version       = AceHealLocals.Version,
    releaseDate   = "",
    aceCompatible = "103",
    author        = "Thirsterhall",
    email         = "",
    website       = "http://",
    category      = "Class",
    db            = AceDatabase:new("AceHealDB"),
    defaults      = AceHealVar.DEFAULT_OPTIONS,
    cmd           = AceChatCmd:new(AceHealLocals.ChatCmd, AceHealLocals.ChatOptions),
})

ace:RegisterFunctions(AceHeal, {
	version = 1.02,
	RetrieveInfoFromTooltip = function(t)
		local d={}
		for i=1,(t:NumLines() or 0),1 do
			tinsert(d,{getglobal(t:GetName().."TextLeft"..i):GetText(),
					   getglobal(t:GetName().."TextRight"..i):GetText()})
		end
		return d
	end,

	SortByMana = function(a,b) 
		if a.mana and b.mana then 
			if a.mana == b.mana then
				return a.rank < b.rank
			else
				return a.mana < b.mana 
			end
		else 
			return true 
		end 
	end,

	ParseByName3 = function(i, p, s)
		local tmpDB= {}
		p = p or "=" s = s or "|" q = q or "\"" i = i..s
        string.gsub(i, "(%a[%a%d_]-)"..p.."("..q.."?)(.-)%2"..s, function(k, _,  v) tmpDB[k] = v end)
        return tmpDB
	end

})

function AceHeal:Initialize()
    -- Helpful closures for accessing the addon's currently loaded profile.
	self.Msg    = function(...) self.cmd:result(format(unpack(arg))) end
	self.Get    = function(var) return self.db:get(self.profilePath, var) end
	self.Set    = function(var, val) self.db:set(self.profilePath, var, val) end
	self.Tog	= function(var) return self.db:toggle(self.profilePath, var) end
	self.SetVar	= function(var, val) if not AceHealVar[var] then AceHealVar[var] = {} end tinsert(AceHealVar[var], val) end
	self.GetVar	= function(var) return AceHealVar[var] end
	self.GetButtonEvent = function(var) return self.db:get({self.profilePath, "ButtonEvents"}, var) end
	self.SetButtonEvent = function(var, val) self.db:set({self.profilePath, "ButtonEvents"}, var, val) end
end

--[[--------------------------------------------------------------------------------
  Addon Enabling/Disabling
-----------------------------------------------------------------------------------]]

function AceHeal:Enable()
	-- needed to update spell data
	self:RegisterEvent("SPELLS_CHANGED", "LoadSpellData")
	-- needed for mana conservation
	self:RegisterEvent("SPELLCAST_START")
	self:RegisterEvent("SPELLCAST_STOP")
	self:RegisterEvent("SPELLCAST_FAILED")
	self:RegisterEvent("SPELLCAST_INTERRUPTED")
	self:RegisterEvent("SPELLCAST_DELAYED")
	-- needed for load on demand addons
	self:RegisterEvent("ADDON_LOADED","AddonLoaded")
	self:RegisterEvent("ACE_PROFILE_LOADED")
	self:LoadBandages()
	self:RegisterEvent("PLAYER_ENTERING_WORLD","LoadBandages");
	self:RegisterEvent("BAG_UPDATE","LoadBandages");
end

-- Disable() is not needed if all you are doing in Enable() is registering events and hooking functions. Ace will automatically unregister and unhook these.
function AceHeal:Disable()
end

--[[--------------------------------------------------------------------------------
  Main Processing
-----------------------------------------------------------------------------------]]
function AceHeal:AddonLoaded()
	if arg1 and AceHealVar.AddonList[arg1] then
		for obj, method in AceHealVar.AddonList[arg1] do
			if obj[method] then obj[method](obj) end
		end
	end
end

function AceHeal:RegisterAddon(obj, addon, method)
	if IsAddOnLoaded(addon) and obj[method] then 
		obj[method or addon](obj)
		return
	end
	if (not AceHealVar.AddonList[addon]) then
		AceHealVar.AddonList[addon]={}
	end
	AceHealVar.AddonList[addon][obj] = (method or addon)
end

function AceHeal:UnregisterAddon(obj, addon)
	if AceHealVar.AddonList[addon] and AceHealVar.AddonList[addon][obj] then
		AceHealVar.AddonList[addon][obj] = nil
	end
end

function AceHeal:ButtonString(button)
	if ( IsControlKeyDown() ) then button = "ctrl_" .. button end
	if ( IsAltKeyDown() )     then button = "alt_" .. button end
	if ( IsShiftKeyDown() )   then button = "shift_" .. button end
	return button
end

function AceHeal:OnClick(button, unit)
	--take care of cursor item if there is one.
--	if self.disabled then return TRUE end
	if button == "LeftButton" and CursorHasItem() then
		if unit == "player" then
			AutoEquipCursorItem()
		else
			DropItemOnUnit(unit)
		end
		return
	end
	-- if a spell is already targeting, cast it on the clicked player and return 
	if SpellIsTargeting() then
		if button == "LeftButton" then 
			SpellTargetUnit(unit)
		elseif button == "RightButton" then
			SpellStopTargeting()
		end
		return 
	end 

	-- run mana conserve check, if the cancel threshold is 0 don't even bother running the function
	if self.Get("cancelthreshold") > 0 then
		if self:ManaConserve(unit) then return end
	end
	
	--if I'm dead then I can do anything
	if not UnitIsDead('player') then
		local btnstring = self:ButtonString(button)
		-- Take care of the dead
	if (UnitIsDead(unit) or UnitIsGhost(unit) or UnitIsCorpse(unit)) then
			if self.Get("ClickRes") and self.GetVar("Resurrection") and btnstring == "LeftButton" then
				self:Res(unit)
				return
			end
		else
			local btnstring = self:ButtonString(button)
			--if I have an action assigned and there isn't already a spell on being targeted
			local event = self.GetButtonEvent(btnstring)
			if event == DBNIL then return TRUE end
			local gevent = getglobal(event)
			local retarget = FALSE
			local r = FALSE
			if UnitIsFriend("player", "target") and unit ~= "target" then 
				retarget = TRUE 
				TargetUnit(unit)
			end
			if type(self[event]) == 'function' then
				self[event](self, unit)
				if SpellIsTargeting() then SpellStopTargeting() end
			elseif type(gevent) == 'function' then
				gevent(unit)
				if SpellIsTargeting() then SpellStopTargeting() end
			else
				r = self:CastSpell(event, unit)
				if SpellIsTargeting() then SpellStopTargeting() end
			end
			if retarget then TargetLastTarget() end
			return r
		end
	end
	return TRUE
end

function AceHeal:CastSpell(spell, unit)
	if UnitExists(unit) == nil then return end
	if self.disabled then return TRUE end
	if not self.GetVar("Load") then self:LoadSpellData() end
	local class = UnitClass("player")
	if AceHealLocals.SpellCat[class][spell] then
		local spell_type = AceHealLocals.SpellCat[class][spell].spelltype
		local lastspell
		if not self.GetVar(spell_type) then return end
		for i, s in self.GetVar(spell_type) do
			-- check mana if I don't have the mana to cast the spell I'm done
			if s.name == spell then
				if s.lvl then
					if s.lvl > (UnitLevel(unit) + 10) then break end
				end
				lastspell = s
			elseif lastspell then 
				break
			end
		end
		if lastspell then
			CastSpell(lastspell.id, SpellBookFrame.bookType)
			if SpellIsTargeting()  then	SpellTargetUnit(unit) end
			return
		end
	else
		CastSpellByName(spell)
		if SpellIsTargeting()  then	SpellTargetUnit(unit) end
		return
	end
	return TRUE
end

function AceHeal:LoadSpellData()
	local i = 1
	if AceHealVar["Efficient"] 		then AceHealVar["Efficient"] = nil end
	if AceHealVar["Emergency"] 		then AceHealVar["Emergency"] = nil end
	if AceHealVar["Instant"] 		then AceHealVar["Instant"] = nil end
	if AceHealVar["Group"] 			then AceHealVar["Group"] = nil end
	if AceHealVar["Resurrection"]	then AceHealVar["Resurrection"] = nil end
	if AceHealVar["Magic"] 			then AceHealVar["Magic"] = nil end
	if AceHealVar["Disease"] 		then AceHealVar["Disease"] = nil end
	if AceHealVar["Poison"] 		then AceHealVar["Poison"] = nil end
	if AceHealVar["Curse"] 			then AceHealVar["Curse"] = nil end
	if AceHealVar["Buff"] 			then AceHealVar["Buff"] = nil end
 while 1 do
	local spell_name, spell_rank = GetSpellName(i, SpellBookFrame.bookType)
		if not spell_name then break end
		if spell_rank then
				_, _, spell_rank = string.find(spell_rank, AceHealLocals.RANK_PATTERN)
				spell_rank = ace.tonum(spell_rank)
		end
		local class = UnitClass("player")
		if AceHealLocals.SpellCat[class][spell_name] then
			local spell_type = AceHealLocals.SpellCat[class][spell_name].spelltype
			AceHealTooltip:SetSpell(i, SpellBookFrame.bookType)
			local tt = self.RetrieveInfoFromTooltip(AceHealTooltip)
	--line 2 left is mana and right is range
			local _, _, m = string.find(tt[2][1], AceHealLocals.MANA_PATTERN)
			if (tt[2][2]) then
				local _, _, rng = string.find(tt[2][2], AceHealLocals.RANGE_PATTERN)
			else
				rng = 0
			end
	--line 3 is the casting time
		local ct
		if tt[3][1] == AceHealLocals.INSTANTCAST then
			ct = 0
		else
			_, _, ct = string.find(tt[3][1], AceHealLocals.CASTTIME_PATTERN)
		end
		if (ct) then
			ct = string.gsub(ct, ",", ".")
		end
	-- spell description or reagent is on line 4 if the spell uses a reagent then the description is on line 5
		local j = 4
		local reag
		if( tt[4] ) then
			if string.sub(tt[j][1],0,string.len(AceHealLocals.REAGENT)) == AceHealLocals.REAGENT then
			reag = string.sub(tt[j][1],string.len(AceHealLocals.REAGENT)+1)
			j=j+1
			end
			else
			j = 3
		end
			local n1, n2, n3, n4
			local _, e, n1 = string.find(tt[j][1], "(%d+[.,]?%d*)")
			if (e) then 
				_, e, n2 = string.find(tt[j][1], "(%d+[.,]?%d*)", e+1)
				if (e) then 
					_, e, n3 = string.find(tt[j][1], "(%d+[.,]?%d*)", e+1)
					if (e) then 
						_, e, n4 = string.find(tt[j][1], "(%d+[.,]?%d*)", e+1)
					end
				end
			end
			local lvl = FALSE
			if AceHealLocals.SpellCat[class][spell_name].lvls then lvl = AceHealLocals.SpellCat[class][spell_name].lvls[spell_rank] end
			self.SetVar(spell_type,{id = i, name = spell_name, rank = spell_rank, casttime = ace.tonum(ct), n1 = ace.tonum(n1), n2 = ace.tonum(n2), n3 = ace.tonum(n3), n4 = ace.tonum(n4), reagent = reag, mana = ace.tonum(m), range = ace.tonum(rng), lvl = lvl })				
		end
		i = i + 1
	end
	if AceHealVar["Efficient"] 		then ace.sort(AceHealVar["Efficient"],self.SortByMana ) end
	if AceHealVar["Emergency"] 		then ace.sort(AceHealVar["Emergency"],self.SortByMana ) end
	if AceHealVar["Instant"] 		then ace.sort(AceHealVar["Instant"],self.SortByMana ) end
	if AceHealVar["Group"] 			then ace.sort(AceHealVar["Group"],self.SortByMana ) end
	if AceHealVar["Resurrection"] 	then ace.sort(AceHealVar["Resurrection"],self.SortByMana ) end
	self.SetVar("Load", TRUE)
end

function AceHeal:LoadBandages()
	AceHealVar["Bandage"] = {}
	local itemlink, code, n1, n2 ,n3, itemid, name
	for b = 0, 4, 1 do
		for s = 1, GetContainerNumSlots(b), 1 do
			itemlink = GetContainerItemLink(b, s)
			if (itemlink) then 
				_,_,name = strfind(itemlink, "|h%[([^]]+)%]|h")
				if name and AceHealLocals.Bandage[name] then 
					self.SetVar("Bandage",{name=name, mana=0, itemlink=itemlink, bag=b, slot=s, n1=ace.tonum(AceHealLocals.Bandage[name].heal)})
				end
			end
		end
	end
end

function AceHeal:UnitInParty(unit)
   if UnitInParty(unit) then return 1 end
   local tmp = strsub(unit, 6, 8)
   if tmp == 'pet' then
      return UnitInParty(strsub(unit, 0, 5)..(strsub(unit, 9, 9)))
   end
   if UnitInRaid(unit) then return 1 end
   tmp = strsub(unit, 5, 7)
   if tmp == 'pet' then
      return UnitInRaid(strsub(unit, 0, 4)..(strsub(unit, 8)))
   end
   return nil
end

function AceHeal:Heal(unit)
	if self.disabled then return end
	self:CastHeal(unit, "Efficient", FALSE)
end

function AceHeal:EmergencyHeal(unit)
	if self.disabled then return end
	self:CastHeal(unit, "Emergency", FALSE)
end

function AceHeal:InstantHeal(unit)
	if self.disabled then return end
	self:CastHeal(unit, "Instant", TRUE)
end

function AceHeal:GroupHeal(unit)
	if self.disabled then return end
	self:CastHeal(unit, "Group", TRUE)
end

function AceHeal:PanicHeal(unit)
	if AceHealLocals.CLASS_DRUID or AceHealLocals.CLASS_SHAMAN then
		SpellStopCasting();
		CastSpellByName(AceHealLocals.NATURESSWIFTNESS);
		SpellStopCasting();
		AceHeal:Heal(unit);
	end
end

function AceHeal:Select(unit)
	if (UnitExists("target") == nil) then
		TargetUnit(unit)
	elseif (UnitIsFriend("target", "player") == nil) then
		TargetUnit(unit)
	else
		TargetUnit(unit)
		ClearTarget()
	end
end

function AceHeal:Res(unit)
	if self.disabled then return end
	if self:UnitDoesntNeedRes(UnitName(unit)) then
		self.Msg("%s is already being resurrected.", UnitName(unit))
	else
		self:CastHeal(unit, "Resurrection", TRUE)
	end
end

function AceHeal:UnitDoesntNeedRes(name)
	-- see who is already ressed by someone else
	if CT_RA_Ressers then
		for key, val in CT_RA_Ressers do
			if val == name then
			return true
			end
		end
	end
	if oRA_Resurrection then
		for key, val in oRA_Resurrection.ressers do
			if val == name then
			return true
			end
		end
	end
	-- see who's ressed already
	if CT_RA_Stats and
		CT_RA_Stats[name] and
		CT_RA_Stats[name]["Ressed"] and
		CT_RA_Stats[name]["Ressed"] == 1 then
		return true
	end
end

function AceHeal:Bandage(unit)
	if self.disabled then return end
	self:CastHeal(unit, "Bandage")
end

function AceHeal:Menu(unit)
	if self.disabled then return end
   local type,x = nil,nil
   if unit == "player" then
      type = PlayerFrameDropDown
   elseif unit == "target" then     
      type = TargetFrameDropDown
   elseif unit == "pet" then
      type = PetFrameDropDown   
   elseif string.find(unit,"party") then
      _,_,x = string.find(unit,"(%d+)")         
      type = getglobal("PartyMemberFrame"..x.."DropDown")
   end   
   if x then this:SetID(x) end
   if type then
      type.unit = unit
      type.name = UnitName(unit)
      ToggleDropDownMenu(1, nil, type,"cursor")
   end
end

function AceHeal:CastHeal(unit, heal_type, best)
	if UnitExists(unit) == nil then return end
	if self.disabled then return end
	if not self.GetVar("Load") then self:LoadSpellData() end
	if not self.GetVar(heal_type) then return end
	local mana = UnitMana('player')
	local health_max, health
	if self:UnitInParty(unit) then
		health_max = UnitHealthMax(unit)
		health = UnitHealth(unit)
	else
		health, health_max = self:GuessUnitHealth(unit)
	end
	health_max = health_max * self.Get("threshold")
	local spell = {}
	local freemana = self:FreeMana()
	if freemana then best = TRUE end
	for i, s in self.GetVar(heal_type) do
		-- check mana if I don't have the mana to cast the spell I'm done
		if s.mana > mana and not freemana then break end
		-- if the spell has levels check against the units level so as not to cast to high a rank
		if s.lvl then
			if s.lvl > (UnitLevel(unit) + 10) then break end
		end
		spell = s
		-- if not using the best(highest) rank check vs units health.
		if not best then
			-- break the search early if we find a smaller sufficient heal
			if s.n1 + health >= health_max then
			   break
			end
		end
	end
	if spell.name then
		if spell.targetself then unit = 'player' end
		-- if we have another friendly unit targeted that is not the clicked player, we need to target the clicked player first
		local retarget = FALSE
		if UnitIsFriend('player', 'target') and not UnitIsUnit('target', unit) then
			retarget = TRUE
			TargetUnit(unit)
		end
		if heal_type == "Bandage" then
			if self.Get("SpellMsg") then 
				self.Msg("Applying %s to %s", spell.name, UnitName(unit)) 
			end
			UseContainerItem( spell.bag, spell.slot )
		else
			if self.Get("SpellMsg") then 
				self.Msg("Cast %s (%s) on %s for %s to %s in %s sec", spell.name, spell.rank, UnitName(unit), spell.n1, spell.n2 or ACE_CMD_REPORT_NO_VAL, spell.casttime or ACE_CMD_REPORT_NO_VAL) 
			end
			CastSpell(spell.id, SpellBookFrame.bookType)
		end
		SpellTargetUnit(unit)
		if spell.casttime or 0 > 0 then
			AceHealVar["casting spell"] = {spell = spell, unit = unit, endTime = GetTime()+(spell.casttime/1000)}
		end
		-- stop targeting just in case we did not apply the spell successfully
		if retarget then TargetLastTarget() end
	end
end

-- interpolates the health of the given unit (used for those outside the party)
function AceHeal:GuessUnitHealth(unit)
	local slope      = ( AceHealVar.ClassHealth[UnitClass(unit)]["60"] - AceHealVar.ClassHealth[UnitClass(unit)]["1"] ) / 60
	local health_max = UnitLevel(unit) * slope + AceHealVar.ClassHealth[UnitClass(unit)]["1"]
	local health     = health_max * (UnitHealth(unit)/UnitHealthMax(unit))
	return health, health_max
end

function AceHeal:FreeMana()
	local i = 1
	while UnitBuff('player', i) do
		AceHealTooltip:SetUnitBuff('player',i)
		for _, b in AceHealLocals.FREE_MANA_BUFFS do
			if AceHealTooltipTextLeft1:GetText() == b then 
				self.Msg("WOOT! Free mana!! Found buff: %s",b)
				return TRUE 
			end
		end
		i = i + 1
	end
	return FALSE
end

function AceHeal:ManaConserve(unit)
	-- check to see if AceHeal is casting a spell right now
	if not AceHealVar["casting spell"] then return end
	if not AceHealVar["casting spell"].started then return end
	arg1 = AceHealVar["casting spell"].unit
	-- don't bother checking until 1 sec left on spell
	if GetTime() + 1000 < AceHealVar["casting spell"].endTime then return end
	local health_max, health
	if self:UnitInParty(arg1) then
		health_max = UnitHealthMax(arg1)
		health = UnitHealth(arg1)
	else
		health, health_max = self:GuessUnitHealth(arg1)
	end
	if (health_max - health ) < AceHealVar["casting spell"].spell.n1 * self.Get("cancelthreshold") then
		if self.Get("SpellMsg") then 
			self.Msg("Spell %s cancled -- %s no longer needs the heal",AceHealVar["casting spell"].spell.name, UnitName(AceHealVar["casting spell"].unit))
		end
		SpellStopCasting()
	if UnitIsUnit(unit,arg1) then
		return TRUE
		end
	end
end

function AceHeal:SPELLCAST_START()
	if AceHealVar["casting spell"] then AceHealVar["casting spell"].started = TRUE end
end

function AceHeal:SPELLCAST_STOP()
	if AceHealVar["casting spell"] then 
		AceHealVar["casting spell"] = FALSE
	end
end

function AceHeal:SPELLCAST_FAILED()
	if AceHealVar["casting spell"] then 
		AceHealVar["casting spell"] = FALSE
	end
end

function AceHeal:SPELLCAST_INTERRUPTED()
	if AceHealVar["casting spell"] then 
		AceHealVar["casting spell"] = FALSE
	end
end

--if spell casting delayed add delayed time to spell end time
function AceHeal:SPELLCAST_DELAYED()
	-- check to see if AceHeal is casting a spell right now
	if not AceHealVar["casting spell"] then return end
	-- check to see if the spell really started
	if not AceHealVar["casting spell"].started then return end
	AceHealVar["casting spell"].endTime = AceHealVar["casting spell"].endTime + (arg1/1000)
end

--[[--------------------------------------------------------------------------------
  Command Handlers
-----------------------------------------------------------------------------------]]
function AceHeal:TogClickToRes()
	self.Tog("ClickRes")
	self.Msg(ACE_CMD_REPORT_LINE, "Click to res now", MAP_ONOFF[self.Get("ClickRes") or 0])
end

function AceHeal:TogSpellMsg()
	self.Tog("SpellMsg")
	self.Msg(ACE_CMD_REPORT_LINE, "Spell messages now", MAP_ONOFF[self.Get("SpellMsg") or 0])
end

function AceHeal:SetThreshold(i)
	i = tonumber(i)
	self.Set("threshold", i)
	self.Msg("Health threshold %s", i)
end

function AceHeal:SetCancelThreshold(i)
	i = tonumber(i)
	self.Set("cancelthreshold", i)
	self.Msg("Spell Cancel threshold %s", i)
end

function AceHeal:SetButton(btnstring)
	if btnstring=="help" then 
		ace:print(AceHealLocals.CHT_BUTTON_HELP)
		return
	end
	local t = self.ParseByName3(btnstring,"=",";")
	for b,v in t do
		if v == "" then v=nil end
		self.SetButtonEvent(b,v)
		self.Msg("%s = %s",b,v or ACE_CMD_REPORT_NO_VAL)
	end
end

function AceHeal:Report()
	if not self.GetVar("Load") then self:LoadSpellData() end
	if (arg1) then
		self:ReportSpellType(arg1)
		return	
	end
	self:ReportSpellType("Efficient")
	self:ReportSpellType("Emergency")
	self:ReportSpellType("Instant")
	self:ReportSpellType("Group")
	self:ReportSpellType("Magic")
	self:ReportSpellType("Disease")
	self:ReportSpellType("Poison")
	self:ReportSpellType("Curse")
	self:ReportSpellType("Resurrection")
	self:ReportSpellType("Buff")
	self:ReportButtons()
end

function AceHeal:ReportButtons()
	local tmpDB = {}
	for k, v in self.Get("ButtonEvents") do
		tinsert(tmpDB,
		 {text = k,
		  val  = v or ACE_CMD_REPORT_NO_VAL})
	end
	self.cmd:report("Button Events", tmpDB)
end

function AceHeal:ReportSpellType(t)
	local tempdb = {}
	if self.GetVar(t) then
		ace:print("Spell type: "..t)
		for _, v in self.GetVar(t) do
			ace:print(format("%s Rank %s Mana:%s Range: %s yrds Reagent: %s cast time: %s, n1=%s, n2=%s, n3=%s, n4=%s, lvl=%s",
				v.name,
				v.rank or ACE_CMD_REPORT_NO_VAL,
				v.mana or ACE_CMD_REPORT_NO_VAL,
				v.range or ACE_CMD_REPORT_NO_VAL,
				v.reagent or ACE_CMD_REPORT_NO_VAL,
				v.casttime or ACE_CMD_REPORT_NO_VAL,
				v.n1 or ACE_CMD_REPORT_NO_VAL,
				v.n2 or ACE_CMD_REPORT_NO_VAL,
				v.n3 or ACE_CMD_REPORT_NO_VAL,
				v.n4 or ACE_CMD_REPORT_NO_VAL,
				v.lvl or ACE_CMD_REPORT_NO_VAL
				))
		end
	end
end

--[[--------------------------------------------------------------------------------
  Register the Addon
-----------------------------------------------------------------------------------]]
AceHeal:RegisterForLoad()