local dewdrop = DewdropLib:GetInstance('1.0')
local tablet = TabletLib:GetInstance('1.0')

AceHealConfigFu = FuBarPlugin:GetInstance("1.2"):new({
	name          = "FuBar - AceHeal Config",
	description   = "A FuBar plugin for configuring AceHeal",
	version       = "0.2",
	releaseDate   = "2006-06-17",
	aceCompatible = 103,
	fuCompatible  = "1.2",
	author        = "Kyahx",
	email         = "Kyahx.Pots@gmail.com",
	category      = "interface", 
	db            = AceDatabase:new("AceHealDB"),
	cmd           = AceChatCmd:new({}, {}),
	
	hasIcon = "Interface\\Icons\\Spell_Holy_SealOfSalvation",
	hasNoText = true,
	cannotDetachTooltip = true,
	hasNoColor = true,
})

function AceHealConfigFu:Initialize()
    -- Helpful closures for accessing the addon's currently loaded profile.
	AceHeal.Get = function(var) return AceHeal.db:get(AceHeal.profilePath, var) end
	AceHeal.Set = function(var, val) AceHeal.db:set(AceHeal.profilePath, var, val) end
	AceHeal.Tog	= function(var) return AceHeal.db:toggle(AceHeal.profilePath, var) end
	AceHeal.GetButtonEvent = function(var) return AceHeal.db:get({AceHeal.profilePath, "ButtonEvents"}, var) end
	AceHeal.SetButtonEvent = function(var, val) AceHeal.db:set({AceHeal.profilePath, "ButtonEvents"}, var, val) end
	
	local ctrl_check
	local alt_check
	local shift_check
end

function AceHealConfigFu:MenuSettings(level, value)
	if(level == 1) then
		dewdrop:AddLine(
			'text', "Assign",
			'hasArrow', true,
			'value', "assign"
		)
		dewdrop:AddLine(
			'text', "Remove",
			'hasArrow', true,
			'value', "remove"
		)
		dewdrop:AddLine() -- Blank line
		dewdrop:AddLine(
			'text', "Click to Ressurect",
			'checked', AceHeal.Get("ClickRes"),
			'func', function() AceHeal.Tog("ClickRes") end
		)
		dewdrop:AddLine(
			'text', "Spell Messages",
			'checked', AceHeal.Get("SpellMsg"),
			'func', function() AceHeal.Tog("SpellMsg") end
		)
		dewdrop:AddLine(
			'text', "Heal Threshold",
			'hasArrow', true,
			'hasSlider', true,
			'sliderValue', tonumber(AceHeal.Get("threshold")),
			'sliderFunc', function(value) AceHeal.Set("threshold", value) end
		)
		dewdrop:AddLine(
			'text', "Cancel Threshold",
			'hasArrow', true,
			'hasSlider', true,
			'sliderBottom', "Off",
			'sliderValue', tonumber(AceHeal.Get("cancelthreshold")),
			'sliderFunc', function(value) AceHeal.Set("cancelthreshold", value) end
		)
		dewdrop:AddLine() -- Blank line
		dewdrop:AddLine(
			'text', "Reset",
			'hasArrow', true,
			'hasEditBox', true,
			'editBoxText', "Type 'CONFIRM' to reset.",
			'editBoxFunc', function(text)
				if text == "CONFIRM" then
					AceHeal.Set("threshold", 0.8)
					AceHeal.Set("cancelthreshold", 0.5)
					AceHeal.Set("ButtonEvents", {})
					AceHeal.Set("SpellMsg", TRUE)
					AceHeal.Set("ClickRes", TRUE)
				end
			end
		)
	elseif(level == 2) then
		if(value == "assign") then
			dewdrop:AddLine(
				'text', "Left Button",
				'hasArrow', true,
				'value', "LeftButton"
			)
			dewdrop:AddLine(
				'text', "Middle Button",
				'hasArrow', true,
				'value', "MiddleButton"
			)
			dewdrop:AddLine(
				'text', "Right Button",
				'hasArrow', true,
				'value', "RightButton"
			)
			dewdrop:AddLine(
				'text', "Button 4",
				'hasArrow', true,
				'value', "Button4"
			)
			dewdrop:AddLine(
				'text', "Button 5",
				'hasArrow', true,
				'value', "Button5"
			)
		elseif(value == "remove") then
			if AceHeal.Get("ButtonEvents") == {} or AceHeal.Get("ButtonEvents") == nil then
				dewdrop:AddLine(
					'text', ("Nothing Assigned")
				)
			else for button, spell in AceHeal.Get("ButtonEvents") do
				local button = button
				dewdrop:AddLine(
					'text', (button.." [|cffFFA500"..spell.."|r]"),
					'func', function()
						AceHeal.SetButtonEvent(button, nil)
						dewdrop:Refresh(level)
					end
				)
			end end
		end
	elseif(level == 3) then
		dewdrop:AddLine(
			'text', "Action",
			'hasArrow', true,
			'hasEditBox', true,
			'editBoxFunc', function(text)
				local button = value
				if ctrl_check then button = "ctrl_"..button end
				if alt_check then button = "alt_"..button end
				if shift_check then button = "shift_"..button end
				AceHeal.SetButtonEvent(button, text)
			end
		)
		dewdrop:AddLine(
			'text', "Ctrl",
			'checked', ctrl_check,
			'func', function()
				if ctrl_check then ctrl_check = nil
				else ctrl_check = 1 end
				dewdrop:Refresh(level)
			end
		)
		dewdrop:AddLine(
			'text', "Alt",
			'checked', alt_check,
			'func', function()
				if alt_check then alt_check = nil
				else alt_check = 1 end
				dewdrop:Refresh(level)
			end
		)
		dewdrop:AddLine(
			'text', "Shift",
			'checked', shift_check,
			'func', function()
				if shift_check then shift_check = nil
				else shift_check = 1 end
				dewdrop:Refresh(level)
			end
		)
	end
end

function AceHealConfigFu:UpdateTooltip()
	tablet:SetTitle(AceHealLocals.Title.." : "..AceHealLocals.Version)
	tablet:SetHint("Right-click to configure AceHeal.")
	local status = tablet:AddCategory(
		'columns', 2
	)
	status:AddLine(
		'text', "Current Profile:",
		'textR', 1,
		'textG', 1,
		'textB', 1,
		'text2', AceHeal.profilePath[2]
	)
end

AceHealConfigFu:RegisterForLoad()