FreeRefills = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0")

function FreeRefills:OnInitialize()
	local defaults = {
	    overstock = false,
		merchant = true,
		bank = true,
		refills = {},
	}

	local args = {
		type = 'group',
		args = {
			add = {
				name = "Add", type = 'text',
			    desc = "Add an item to the refill list. (Usage: /freerefills add [Item Link] #)",
				get = false,
			    set = function(text) self:AddRefillItem(text) end,
				usage = "[Item Link] <number of items>",
				guiHidden = true
			},
			addstack = {
				name = "Add Stack", type = 'text',
			    desc = "Add an item to the refill list by stack. (Usage: /freerefills addstack [Item Link] #)",
			    get = false,
				set = function(text) self:AddRefillItemStack(text) end,
				usage = "[Item Link] <number of stacks>",
				guiHidden = true
			},
			del = {
				name = "Remove", type = 'text',
			    desc = "Remove an item from the refill list. (Usage: /freerefills del [Item Link])",
				get = false,
			    set = function(text) self:RemoveRefillItem(text) end,
				usage = "[Item Link]",
				guiHidden = true
			},
			overstock = {
				name = "Overstock", type = 'toggle',
				desc = "Toggle Overstocking.  If an item is sold in stacks and cannot meet your exact requested amount, you can opt to buy slightly more (Overstock).",
				get = function() return self.db.profile.overstock end,
				set = function() self.db.profile.overstock = not self.db.profile.overstock end
			},
			merchant = {
				name = "Process Merchant", type = 'toggle',
				desc = "Toggle Merchant Looting.",
				get = function() return self.db.profile.merchant end,
				set = function()
					if self.db.profile.merchant == true then
						self:UnregisterEvent("MERCHANT_SHOW")
						self.db.profile.merchant = false
					elseif self.db.profile.merchant == false then
						self:RegisterEvent("MERCHANT_SHOW")
						self.db.profile.merchant = true
					end
				end
			},
			bank = {
				name = "Process Bank", type = 'toggle',
				desc = "Toggle Bank Looting.",
				get = function() return self.db.profile.bank end,
				set = function()
					if self.db.profile.bank == true then
						self:UnregisterEvent("BANKFRAME_OPENED")
						self.db.profile.bank = false
					elseif self.db.profile.bank == false then
						self:RegisterEvent("BANKFRAME_OPENED")
						self.db.profile.bank = true
					end
				end
			},
			reset = {
				name = "Clear", type = 'execute',
			    desc = "Clear the refill list completely.",
			    func = function() self:ClearRefills() end
			},
			list = {
				name = "List", type = 'execute',
			    desc = "List the items in the refill list.",
			    func = function() self:List() end
			}
		}
	}
	
	self:RegisterDB("FreeRefillsDB")
	self:RegisterDefaults('profile', defaults )
	
	self:RegisterChatCommand({ "/freerefills", "/fr" }, args )
	
	if not self.db.char.notFirstTime then 
		self.db.char = {} -- clean out the old data from my non-profile usage
		self.db.char.notFirstTime = true
		self:SetProfile('char')
	end
	
	--Copy and Nuke old Ace1 SavedData
	local realm,char = GetRealmName(), UnitName("player")
	if FreeRefillsDB[realm..':'..char] then
		for k,v in pairs(FreeRefillsDB[realm..':'..char]) do
			if k == "mood" then
				if v == "Pessimest" then self.db.profile.overstock = true
				else self.db.profile.overstock = false end
			else
				self.db.profile.refills[k] = v
			end
		end
		if FreeRefillsDB[realm..':'..char] then FreeRefillsDB[realm..':'..char] = nil end
	end
end

function FreeRefills:OnEnable()
	if self.db.profile.merchant then self:RegisterEvent("MERCHANT_SHOW") end
	if self.db.profile.bank then self:RegisterEvent("BANKFRAME_OPENED") end
end

function FreeRefills:OnDisable()
	if self.db.profile.merchant then self:UnregisterEvent("MERCHANT_SHOW") end
	if self.db.profile.bank then self:UnregisterEvent("BANKFRAME_OPENED") end
end

function FreeRefills:debug(msg)
--	self:Print("|cff0099CC"..msg.."|r")
end

--[[--------------------------------------------------------------------------------
  Merchant Processing
-----------------------------------------------------------------------------------]]

function FreeRefills:MERCHANT_SHOW()
	self:debug("Merchant Opened")
	for itemID,amount in pairs(self.db.profile.refills) do
		local merchSlot = self:ItemInStock(itemID)
		if merchSlot then
			self:debug("Item Found in Merchant Slot #"..merchSlot)
			local amountInBags = self:CountItemInBags(itemID)
			if amountInBags >= amount then
				self:debug("Already have enough in bags.")
			else
				local stackCycle, stackRem = self:CalculateNeeded(amount, amountInBags, merchSlot)
				if stackCycle then self:debug("Need to buy "..stackCycle.." more full stacks.") end
				if stackRem then self:debug("Need to buy "..stackRem.." item(s) (Remainder stack)") end
				local _, _, _, _, numAvailable = GetMerchantItemInfo(merchSlot)
				if stackCycle then
					if stackCycle*GetMerchantItemMaxStack(merchSlot) > numAvailable and numAvailable > 0 then
						self:debug("Can only buy "..numAvailable)
						BuyMerchantItem(merchSlot, numAvailable)
					else
						for i = 1, stackCycle do
						BuyMerchantItem(merchSlot, GetMerchantItemMaxStack(merchSlot))
					end
				end
			end
			if stackRem then
				if stackRem > numAvailable and numAvailable > 0 then
					self:debug("Can only buy "..numAvailable)
					stackRem = numAvailable
				end
				BuyMerchantItem(merchSlot, stackRem)
				end
			end
		end
	end
end

function FreeRefills:ItemInStock(itemID)
	self:debug("Searching for Item: "..itemID)
	local NumItems = GetMerchantNumItems()
	for i = 1, NumItems do
		local slotLinkString = GetMerchantItemLink(i)
		local slotID = self:ExtractItemID(slotLinkString)
		if slotID == itemID then
			return i
		end
	end
end

function FreeRefills:CalculateNeeded(amount, amountInBags, merchSlot)
	-- more detailed logic regauding stacks should be taken care of here
	local amountNeeded
	local _, _, _, stackSize = GetMerchantItemInfo(merchSlot)
	local stackedAmount
	local stackedNeeded
	local maxBuy = GetMerchantItemMaxStack(merchSlot)
	local stackCycle
	local stackRem
	
	self:debug("Stacksize: "..stackSize)
	
	if stackSize ~= 1 then
		stackedAmount = amount / stackSize
		stackedNeeded = amountInBags / stackSize
		amountNeeded = stackedAmount - stackedNeeded
		self:debug("Unrounded Needed: "..amountNeeded)
		if self.db.profile.overstock == true then
			stackCycle = self:RoundNumberUp(amountNeeded)
		else
			stackCycle = self:RoundNumberDown(amountNeeded)
		end
	else
		local finalNeeded = amount - amountInBags
		self:debug("Need to buy "..finalNeeded.." more.  Merchant sells in max stacks of "..maxBuy..".")
		if finalNeeded > maxBuy then
			local stacks = finalNeeded / maxBuy
			stackCycle = self:RoundNumberDown(stacks)
			stackRem = finalNeeded - (stackCycle * maxBuy)
		else
			stackRem = finalNeeded
		end
	end

	if stackCycle and stackCycle <= 0 then stackCycle = nil end
	if stackRem and stackRem <= 0 then stackRem = nil end
	
	return stackCycle, stackRem
end

function FreeRefills:RoundNumberDown(num)
	local roundNum = tonumber(string.format("%.".."f", num ))
	return roundNum
end

function FreeRefills:RoundNumberUp(num)
	local roundNum = tonumber(string.format("%.".."f", num ))
	if roundNum ~= num then roundNum = roundNum + 1 end
	return roundNum
end

--[[--------------------------------------------------------------------------------
  Bank Processing
-----------------------------------------------------------------------------------]]

function FreeRefills:BANKFRAME_OPENED()
	self:debug("Bank Opened")
	for itemID,amount in pairs(self.db.profile.refills) do
		local amountInBank = self:CountItemInBank(itemID)
		if amountInBank > 0 then
			local amountInBags = self:CountItemInBags(itemID)
			local amountNeeded = amount - amountInBags
			if amountInBags >= amount then
				self:debug("Already have enough in bags.")
			else
				if amountNeeded > amountInBank then
					amountNeeded = amountInBank end
				self:debug(amountNeeded.." item(s) to be looted.")
				self:LootBank(itemID, amountNeeded)
			end
		end
	end
end

function FreeRefills:CountItemInBank(itemID)
	self:debug("Searching for Item: "..itemID)
	local itemCount = 0
	-- Scan the main bank first
	for slot = 1, 24 do
		local slotID = self:ExtractItemID(GetContainerItemLink(-1,slot))
		if slotID == itemID then
			local _,slotCount = GetContainerItemInfo(-1, slot)
			itemCount = itemCount + slotCount
		end
	end
	--Now Scan bank bags
	for bag = 5, 10, 1 do
		local size = GetContainerNumSlots(bag);
		if (size > 0) then
			for slot = 1, size, 1 do
				local slotID = self:ExtractItemID(GetContainerItemLink(bag,slot));
				if slotID == itemID then
					local _,slotCount = GetContainerItemInfo(bag, slot);
					itemCount = itemCount + slotCount;
				end            
			end
		end
	end
	self:debug(itemCount.." item(s) found in player's bank")
	return itemCount
end

function FreeRefills:LootBank(itemID, amountNeeded)
	self:debug("Bank Looting Logic: #"..itemID.." - "..amountNeeded)
	--Scan the main bank first
	for slot = 1, 24 do
		local slotID = self:ExtractItemID(GetContainerItemLink(-1,slot))
		if slotID == itemID then
			local _,slotCount = GetContainerItemInfo(-1, slot)
			self:debug(slotCount.." Item(s) Found in Bank:Slot "..slot)
			if slotCount <= amountNeeded then --We don't have to split this stack, we can cheat and use UseContainerItem
				self:debug("Looting Entire Stack")
				UseContainerItem(-1, slot)
				amountNeeded = amountNeeded - slotCount
				self:debug(slotCount.." Looted - Need "..amountNeeded.." more item(s)")
			else
				SplitContainerItem(-1, slot, amountNeeded)
				for bag = 4, 0, -1 do
					local size = GetContainerNumSlots(bag);
					if (size > 0) then
						for slot = 1, size, 1 do
							local slotID = self:ExtractItemID(GetContainerItemLink(bag,slot));
							if not slotID then
								if bag == 0 then
									PutItemInBackpack()
								else
									local bagID
									local bag = tonumber(bag)
									bagID = bag + 19
									PutItemInBag(bagID)
								end
							end
						end
					end
				end
				local amountLooted = amountNeeded
				amountNeeded = amountNeeded - amountLooted -- This should always result in 0
				self:debug(amountLooted.." Looted - Need "..amountNeeded.." more item(s)")
			end
		end
		if amountNeeded == 0 then self:debug(amountNeeded.." item(s) # "..itemID.." needed.  Next item.") return end
	end
	self:debug("Main Bank didn't cover it, scanning additional Bags")
	--Now Scan bank bags
	for bag = 5, 10, 1 do
		local size = GetContainerNumSlots(bag);
		if (size > 0) then
			for slot = 1, size do
				local slotID = self:ExtractItemID(GetContainerItemLink(bag,slot))
				if slotID == itemID then
					local _,slotCount = GetContainerItemInfo(bag, slot)
					self:debug(slotCount.." Item(s) Found in Bag "..bag..":Slot "..slot..". Need to loot "..amountNeeded)
					if slotCount <= amountNeeded then --We don't have to split this stack, we can cheat and use UseContainerItem
						self:debug("Looting Entire Stack")
						UseContainerItem(bag, slot)
						amountNeeded = amountNeeded - slotCount
						self:debug(slotCount.." Looted - Need "..amountNeeded.." more item(s)")
					else
						SplitContainerItem(bag, slot, amountNeeded)
						for bag = 4, 0, -1 do
							local size = GetContainerNumSlots(bag);
							if (size > 0) then
								for slot = 1, size, 1 do
									local slotID = self:ExtractItemID(GetContainerItemLink(bag,slot));
									if (not slotID) then
										if bag == 0 then
											PutItemInBackpack()
										else
											local bagID
											local bag = tonumber(bag)
											bagID = bag + 19
											PutItemInBag(bagID)
										end
									end
								end
							end
						end
						local amountLooted = amountNeeded
						amountNeeded = amountNeeded - amountLooted -- This should always result in 0
						self:debug(amountLooted.." Looted - Need "..amountNeeded.." more item(s)")
					end
				end
			if amountNeeded == 0 then self:debug(amountNeeded.." item(s) # "..itemID.." needed.  Next item.") return end
			end
		end
	end
end

--[[--------------------------------------------------------------------------------
  Shared Functions
-----------------------------------------------------------------------------------]]

function FreeRefills:CountItemInBags(itemID)
	local itemCount = 0
	for bag = 4, 0, -1 do
		local size = GetContainerNumSlots(bag);
		if (size > 0) then
			for slot = 1, size, 1 do
				local slotID = self:ExtractItemID(GetContainerItemLink(bag,slot));
				if slotID == itemID then
					local _,slotCount = GetContainerItemInfo(bag, slot);
					itemCount = itemCount + slotCount;
				end            
			end
		end
	end
	self:debug(itemCount.." item(s) found in player's bags")
	return itemCount
end

function FreeRefills:ExtractItemID(linkstring)
	if linkstring ~= nil then
		local _, _, itemID = string.find(linkstring,"-*:(%d+):.*")
		return itemID
	end
end

--[[--------------------------------------------------------------------------------
  Command Handlers
-----------------------------------------------------------------------------------]]

function FreeRefills:AddRefillItem(msg)
	--add an item to the refill list
	local _, _, link, num = string.find(msg, "(.+)%s(%d+)")
	local itemID = self:ExtractItemID(link)
	num = tonumber(num)
	self.db.profile.refills[itemID] = num
	self:Print("Added Item ID#"..itemID.." - Amount:"..num)
end

function FreeRefills:AddRefillItemStack(msg)
	--add an item to the refill list by stack
	local _, _, link, num = string.find(msg, "(.+)%s(%d+)")
	local itemID = self:ExtractItemID(link)
	local _, _, _, _, _, _, itemStack = GetItemInfo(itemID)
	num = tonumber(num)
	num = num * itemStack
	self.db.profile.refills[itemID] = num
	self:Print("Added Item ID#"..itemID.." - Amount:"..num)
end

function FreeRefills:RemoveRefillItem(msg)
	--remove an item from the refill list
	local itemID = self:ExtractItemID(msg)
	self.db.profile.refills[itemID] = nil
	self:Print("Removed Item ID#"..itemID)
end

function FreeRefills:ClearRefills()
	--clear the refill list
	self.db.profile.refills = {}
	self:Print("Item List Cleared")
end

function FreeRefills:List()
	--report back which items are currently on the refill list, and their quanitity
	self:Print("Refill List: ")
	local notempty
	for itemID,amount in pairs(self.db.profile.refills) do
		local name = GetItemInfo(itemID)
		if name ~= nil then
			self:Print("     "..name.." ("..amount..")")
			notempty = 1
		end
	end
	if not notempty then self:Print("     None") end
end
