local loc = ItemBonusesFu.locals
local const = ItemBonusesFu.constants
local tablet = TabletLib:GetInstance('1.0')
local dewdrop = DewdropLib:GetInstance('1.0')
local compost = CompostLib:GetInstance('compost-1')
local crayon = CrayonLib:GetInstance("1.0")

compost:Reclaim(ItemBonusesFu)
ItemBonusesFu = nil

ItemBonusesFu = FuBarPlugin:GetInstance("1.2"):new({
   name          = const.Title,
   description   = loc.Desc,
   version       = "0.4.3",
   releaseDate   = string.sub("$Date: 2006-05-22 09:56:23 -0400 (Mon, 22 May 2006) $", 8, 17),
   author        = "Prandur",
   website       = "http://prandur.wowinterface.com",
   aceCompatible = 103,
   category      = "inventory",
   db            = AceDatabase:new("FuBar_ItemBonusesDB"),
   cmd           = AceChatCmd:new(const.Cmd, const.Cmd_Opt),
   hasIcon       = "Interface\\Icons\\Spell_Nature_EnchantArmor.blp",
   canHideText   = true,
})


function ItemBonusesFu:Enable()
	self:Hook("BonusScanner_Update", "Update")
end

function ItemBonusesFu:Disable()
end

function ItemBonusesFu:UpdateData()
	self:debug("Updating Data")
	local check
	for i,e in ipairs(const.ITEMBONUSES_EFFECTS) do
		if self.data[e.effect] then
			check = true
		end
	end
	if not check then
		self.data.display_none = true
	end
end

function ItemBonusesFu:UpdateText()
	self:debug("Updating Text")
	if self.data.display_none then
		if self.data.short_display then
			self:SetText(loc.SHORT_TEXT)
		else
			self:SetText(loc.TEXT)
		end
	else
		if(not BonusScanner or not BonusScanner.active) then
			self:SetText(crayon:Red(loc.BONUSSCANNER_MISSING))
		else
			local text = compost:Acquire()
			for i,e in ipairs(const.ITEMBONUSES_EFFECTS) do
				if self.data[e.effect] and BONUSSCANNER_NAMES[e.effect] then
					table.insert(text, self:GetBonusText(e, false))
				end
			end
			self:SetText(table.concat(text, " "))
			compost:Reclaim(text)
		end
	end
end

function ItemBonusesFu:UpdateTooltip()
	self:debug("Updating Tooltip")
	if (not BonusScanner or not BonusScanner.active) then
		local TabCat = tablet:AddCategory(
			'columns', 1,
			'child_textR', 1,
			'child_textG', 0,
			'child_textB', 0
		)
		TabCat:AddLine('text', loc.BONUSSCANNER_MISSING)
	else
		local TabCat = compost:Acquire()
		
		for i,itemcat in ipairs(const.ITEMBONUSES_CATEGORIES) do
			TabCat[itemcat] = tablet:AddCategory(
				'text', loc.CAT[itemcat],
				'columns', 2,
				'textR', 0,
				'textG', 1,
				'textB', 0,
				'child_textR', 1,
				'child_textG', 1,
				'child_textB', 0,
				'child_text2R', 1,
				'child_text2G', 1,
				'child_text2B', 1
			)
		end
		
		for i,e in ipairs(const.ITEMBONUSES_EFFECTS) do
			if BonusScanner.bonuses[e.effect] and BONUSSCANNER_NAMES[e.effect] then
				local val
				if e.format then
		   			val = format(e.format,BonusScanner.bonuses[e.effect])
				else
					val = BonusScanner.bonuses[e.effect]
				end
				TabCat[e.cat]:AddLine(
					'text', BONUSSCANNER_NAMES[e.effect],
					'text2', val
				)
			end
		end

		compost:Reclaim(TabCat)
	end
end

function ItemBonusesFu:MenuSettings(level, value)
	if level == 1 then
		dewdrop:AddLine(
			'text', loc.DISPLAY_NONE,
			'func', function() self:ToggleOption("display_none", true) end,
			'checked', self.data.display_none
		)
		for  i,e in ipairs(const.ITEMBONUSES_CATEGORIES) do
			local my_e = e
			dewdrop:AddLine(
				'text', loc.CAT[my_e],
				'hasArrow', true,
				'value', my_e
			)
		end
		dewdrop:AddLine()
		dewdrop:AddLine(
			'text', loc.SHORTDISPLAY,
			'func', function() self:ToggleOption("short_display", true) end,
			'checked', self.data.short_display
		)
		dewdrop:AddLine(
			'text', loc.COLORED,
			'func', function() self:ToggleOption("colored", true) end,
			'checked', self.data.colored
		)
	elseif level == 2 then
		for i,e in ipairs(const.ITEMBONUSES_EFFECTS) do
			if e.cat == value and BONUSSCANNER_NAMES[e.effect] then
				local my_e = e
				dewdrop:AddLine(
					'text', self:GetBonusText(my_e, true),
					'func', function() self:ToggleOption(my_e.effect, true) end,
					'checked', self.data[my_e.effect]
				)
			end
		end
	end
end

function ItemBonusesFu:ToggleOption(var, doUpdate)
	self.data[var] = not self.data[var]
	if doUpdate then
		self:Update()
	end
	return self.data[var]
end

function ItemBonusesFu:GetBonusText(EffectTable, forMenu)
	local t = compost:Acquire()
	local val
	if BonusScanner.bonuses[EffectTable.effect] then
		val = format(EffectTable.format, BonusScanner.bonuses[EffectTable.effect])
	end
	if forMenu then
		table.insert(t, "[" .. crayon:Colorize(const.ItemBonuses_colors[EffectTable.color], EffectTable.short) .. "] " .. BONUSSCANNER_NAMES[EffectTable.effect])
	else
		if self.data.short_display then
			table.insert(t, crayon:Colorize(const.ItemBonuses_colors[EffectTable.color], EffectTable.short))
		else
			table.insert(t, crayon:Colorize(const.ItemBonuses_colors[EffectTable.color], BONUSSCANNER_NAMES[EffectTable.effect]))
		end
	end
	if not val then
		if not forMenu then
			val = format(EffectTable.format, 0)
		else
			local text = table.concat(t)
			compost:Reclaim(t)
			return text
		end
	end
	if self.data.colored then
		val = crayon:Colorize(const.ItemBonuses_colors[EffectTable.color], val)
	else
		val = crayon:White(val)
	end
	if forMenu then
		table.insert(t, " (" .. val .. ")")
	else
		table.insert(t, 1, " ")
		table.insert(t, val)
	end
	local text = table.concat(t)
	compost:Reclaim(t)
	return text
end


ItemBonusesFu:RegisterForLoad()