local compost = CompostLib:GetInstance('compost-1')
local dewdrop = DewdropLib:GetInstance('1.0')
local tablet = TabletLib:GetInstance('1.0')

FuBar_Regen = FuBarPlugin:GetInstance("1.2"):new({
	name          = FuBar_RegenLocals.NAME,
	description   = FuBar_RegenLocals.DESCRIPTION,
	version       = "1.2.1",
	releaseDate   = "05-28-2006",
	aceCompatible = 103,
	author        = "Rophy",
	email         = "rophy123@gmail.com",
	website       = "http://www.wowinterface.com/downloads/fileinfo.php?id=4717",
	category      = "combat",
	db            = AceDatabase:new("FuBar_RegenDB"),
	defaults      = {
    showHP = false,
    showMP = false,
    showLabel = true,
		showColor = true,
		showPercent = false,
	},
	loc           = FuBar_RegenLocals,
	hasIcon       = false,

	-- Methods
	IsShowHP = function(self)
		return self.data.showHP;
	end,
	
	IsShowMP = function(self)
		return self.data.showMP;
	end,
	
	IsShowCurrent = function(self)
		return self.data.showCurrent;
	end,
	
	IsShowPercent = function(self)
		return self.data.showPercent;
	end,
		
	
	IsHideLabel = function(self)
		return self.data.hideLabel;
	end,
	
	ToggleShowHP = function(self)
		self.data.showHP = not self.data.showHP;
		self:UpdateSettings();
	end,
	
	ToggleShowMP = function(self)
		self.data.showMP = not self.data.showMP;
		self:UpdateSettings();
	end,

	ToggleShowCurrent = function(self)
		self.data.showCurrent = not self.data.showCurrent;
		self:UpdateSettings();
	end,
		
	ToggleShowPercent = function(self)
		self.data.showPercent = not self.data.showPercent;
		self:Update();
	end,
	
	
	ToggleHideLabel = function(self)
		self.data.hideLabel = not self.data.hideLabel;
		self:Update();
	end,
	
	Enable = function(self)
		self:RegisterEvent("UNIT_HEALTH", "OnUnitHealth");
		self:RegisterEvent("UNIT_MANA", "OnUnitMana");
		self:RegisterEvent("PLAYER_ENTERING_WORLD", "OnPlayerEnteringWorld");
		self:RegisterEvent("PLAYER_REGEN_DISABLED", "OnPlayerRegenDisabled");
		self:RegisterEvent("PLAYER_REGEN_ENABLED", "OnPlayerRegenEnabled");	

	end,
	

	
	MenuSettings = function(self, level, value)
		if level == 1 then
			dewdrop:AddLine(
				'text', self.loc.MENU_SHOW_HP,
				'arg1', self,
				'func', 'ToggleShowHP',
				'checked', self:IsShowHP()
			);
			
			dewdrop:AddLine(
				'text', self.loc.MENU_SHOW_MP,
				'arg1', self,
				'func', 'ToggleShowMP',
				'checked', self:IsShowMP()
			);
			dewdrop:AddLine(
				'text', self.loc.MENU_SHOW_PERCENT,
				'arg1', self,
				'func', 'ToggleShowPercent',
				'checked', self:IsShowPercent()
			);
			dewdrop:AddLine(
				'text', self.loc.MENU_SHOW_CURRENT,
				'arg1', self,
				'func', 'ToggleShowCurrent',
				'checked', self:IsShowCurrent()
			);
			dewdrop:AddLine(
				'text', self.loc.HIDE_LABEL,
				'arg1', self,
				'func', 'ToggleHideLabel',
				'checked', self:IsHideLabel()
			);
			
		end
	end,
	
	vars = {
		currHealth = 0,
		currMana = 0,
		regenHP = 0,
		regenMP = 0,
		checkedManaState = 0,
		maxHPRate = 0,
		minHPRate = 9999,
		maxMPRate = 0,
		minMPRate = 9999,
		regenCombatTrack = 0,
		regenMPDuringCombat = 0,
		regenHPDuringCombat = 0,
	},
		
	
	OnPlayerEnteringWorld = function(self)
		if UnitManaMax("player") == 0 then
			self.data.showMP = FALSE;
		end
	end,
	
	OnPlayerRegenDisabled	= function(self)
		self.vars.regenCombatTrack = 1;
		self.vars.regenMPDuringCombat = 0;
		self.vars.regenHPDuringCombat = 0;
	end,

	OnPlayerRegenEnabled	= function(self)
		self.vars.regenCombatTrack = 0;	
	end,

	OnUnitHealth = function(self)
		if self:IsShowHP() then
			local currHealth = UnitHealth("player");
			if ( currHealth > self.vars.currHealth and self.vars.currHealth  ~= 0 ) then
				self.vars.regenHP  = currHealth - self.vars.currHealth ;
				
				if ( self.vars.regenCombatTrack == 1) then
					self.vars.regenHPDuringCombat = self.vars.regenHPDuringCombat + self.vars.regenHP;
				end 
				
				if (self.vars.regenHP > self.vars.maxHPRate) then 
					self.vars.maxHPRate = self.vars.regenHP ;
				end
				if (self.vars.regenHP < self.vars.minHPRate or self.vars.minHPRate == 9999) then 
					self.vars.minHPRate = self.vars.regenHP;
				end				
			end
			self.vars.currHealth = currHealth;
			self:Update();
		end
	end,
	
	OnUnitMana = function(self)
		if self:IsShowMP() then
			local currMana = UnitMana("player");
			if ( currMana  > self.vars.currMana and self.vars.currMana ~= 0 ) then
				self.vars.regenMP  = currMana - self.vars.currMana  ;

				if ( self.vars.regenCombatTrack == 1) then
					self.vars.regenMPDuringCombat = self.vars.regenMPDuringCombat + self.vars.regenMP;
				end 

				if (self.vars.regenMP > self.vars.maxMPRate) then 
					self.vars.maxMPRate = self.vars.regenMP;
				end
				if (self.vars.regenMP < self.vars.minMPRate or self.vars.minMPRate == 9999) then 
					self.vars.minMPRate = self.vars.regenMP;
				end								
			end
			self.vars.currMana = currMana;
			self:Update();
		end
	end,
	
	UpdateText = function(self)
		local labelTextHP = "";
		local valueTextHP = "";
		local labelTextMP = "";
		local valueTextMP = "";
		
		if UnitHealth("player") == UnitHealthMax("player") then
			self.vars.regenHP = 0;
		end
		if UnitMana("player") == UnitManaMax("player") then
			self.vars.regenMP = 0;
		end	
				
		-- safety in case both are off, then cant ever turn em on
		if ( not self:IsShowHP() and not self:IsShowMP() ) then
			self:ToggleShowHP();
		end
		
		if ( self:IsShowHP() ) then
			if not self:IsHideLabel() then
				labelTextHP = self.loc.HP_LABEL;
			end
			
			if self:IsShowPercent() then
				valueTextHP = format("%.2f%%", (self.vars.regenHP/UnitHealthMax("player"))*100);
			else
				valueTextHP = format("%d", self.vars.regenHP);	
			end	

			valueTextHP = FuBarUtils.Green(valueTextHP);

			if self:IsShowCurrent() then
				valueTextHP = format("%s/%s(%s)", FuBarUtils.Green(UnitHealth("player")), FuBarUtils.Green(UnitHealthMax("player")), valueTextHP)
			end
			


		end
		
		if ( self:IsShowMP() ) then
			if not self:IsHideLabel() then
				labelTextMP = self.loc.MP_LABEL;
			end
			
			if self:IsShowPercent() then
				valueTextMP = format("%.2f%%", (self.vars.regenMP/UnitManaMax("player"))*100);
			else
				valueTextMP = format("%d", self.vars.regenMP);	
			end
			
			valueTextMP = FuBarUtils.Colorize("3399ff", valueTextMP);

			if self:IsShowCurrent() then
				valueTextMP = format("%s/%s(%s)", FuBarUtils.Colorize("3399ff", UnitMana("player")), FuBarUtils.Colorize("3399ff", UnitManaMax("player")), valueTextMP)
			end

		end
		
		
		local t = compost:Acquire()
		table.insert(t, labelTextHP..valueTextHP.." "..labelTextMP..valueTextMP);
		self:SetText(table.concat(t, " "))
		compost:Reclaim(t);
	end,
	
	UpdateTooltip = function(self)
		local minHP = self.vars.minHPRate;
		local minMP = self.vars.minMPRate;
		
		if minHP == 9999 then minHP = 0 end;
		if minMP == 9999 then minMP = 0 end;	

		local cat, regenPercent
		
		if self:IsShowHP() then
		
			regenPercent = (self.vars.regenHPDuringCombat/UnitHealthMax("player"))*100;
			
			cat = tablet:AddCategory(
				'columns', 2,
				'child_textR', 1,
				'child_textG', 1,
				'child_textB', 0,
				'child_text2R', 0,
				'child_text2G', 1,
				'child_text2B', 0
			);

			cat:AddLine(
				'text', self.loc.TOOLTIP1_LEFT, 
				'text2', string.format("%s |cffffffff/|r %s |cffffffff(|r|cffff0000%s|cffffffff)|r", UnitHealth("player"), UnitHealthMax("player"), UnitHealthMax("player")-UnitHealth("player") )
			);
			cat:AddLine(
				'text', self.loc.TOOLTIP3_LEFT, 
				'text2', self.vars.maxHPRate
			);
			cat:AddLine(
				'text', self.loc.TOOLTIP4_LEFT, 
				'text2', minHP
			);
			cat:AddLine(
				'text', self.loc.TOOLTIP8_LEFT, 
				'text2', format("%s (%.2f%%)", self.vars.regenHPDuringCombat, regenPercent )
			);			
		end
		
		if self:IsShowMP() then
		
			regenPercent = (self.vars.regenMPDuringCombat/UnitManaMax("player"))*100;
			
			cat = tablet:AddCategory(
				'columns', 2,
				'child_textR', 1,
				'child_textG', 1,
				'child_textB', 0,
				'child_text2R', 0.2,
				'child_text2G', 0.4,
				'child_text2B', 1
			);

			cat:AddLine(
				'text', self.loc.TOOLTIP2_LEFT, 
				'text2', string.format("%s |cffffffff/|r %s |cffffffff(|r|cffff0000%s|cffffffff)|r", UnitMana("player"), UnitManaMax("player"), UnitManaMax("player")-UnitMana("player") )
			);
			cat:AddLine(
				'text', self.loc.TOOLTIP5_LEFT, 
				'text2', self.vars.maxMPRate
			);
			cat:AddLine(
				'text', self.loc.TOOLTIP6_LEFT, 
				'text2', minMP
			);
			cat:AddLine(
				'text', self.loc.TOOLTIP7_LEFT, 
				'text2', format("%s (%.2f%%)", self.vars.regenMPDuringCombat, regenPercent )
			);
					
		end		

	end,
	
	UpdateSettings = function(self)
		-- safety in case both are off, then cant ever turn em on
		if ( not self:IsShowHP() and not self:IsShowMP() ) then
			self:ToggleShowHP();

		end
		self:Update();
	end,
		

})


FuBar_Regen:RegisterForLoad()