local dewdrop = DewdropLib:GetInstance('1.0')
local tablet = TabletLib:GetInstance('1.0')

local trackerSpells = {
	FuBar_TrackerLocals.TRACKER_SPELL_FIND_HERBS,
	FuBar_TrackerLocals.TRACKER_SPELL_FIND_MINERALS,
	FuBar_TrackerLocals.TRACKER_SPELL_FIND_TREASURE,
	FuBar_TrackerLocals.TRACKER_SPELL_TRACK_BEASTS,
	FuBar_TrackerLocals.TRACKER_SPELL_TRACK_HUMANOIDS,
	FuBar_TrackerLocals.TRACKER_SPELL_TRACK_HIDDEN,
	FuBar_TrackerLocals.TRACKER_SPELL_TRACK_ELEMENTALS,
	FuBar_TrackerLocals.TRACKER_SPELL_TRACK_UNDEAD,
	FuBar_TrackerLocals.TRACKER_SPELL_TRACK_DEMONS,
	FuBar_TrackerLocals.TRACKER_SPELL_TRACK_GIANTS,
	FuBar_TrackerLocals.TRACKER_SPELL_TRACK_DRAGONKIN,
	FuBar_TrackerLocals.TRACKER_SPELL_SENSE_UNDEAD,
	FuBar_TrackerLocals.TRACKER_SPELL_SENSE_DEMONS,
}

local trackerLookUp = {}

FuBar_Tracker = FuBarPlugin:GetInstance("1.2"):new({
	name          = FuBar_TrackerLocals.NAME,
	description   = FuBar_TrackerLocals.DESCRIPTION,
	version       = "1.0.4",
	releaseDate   = "06-13-2006",
	aceCompatible = 103,
	author        = "Corgi",
	email         = "corgiwow@gmail.com",
	website       = "http://corgi.wowinterface.com/",
	category      = "interface", -- Set this to the appropriate category.
	db            = AceDatabase:new("FuBar_TrackerDB"),
	defaults      = {
		showMiniMap = FALSE,
		currTexture = nil,
	},
	cmd           = AceChatCmd:new(FuBar_TrackerLocals.COMMANDS, FuBar_TrackerLocals.CMD_OPTIONS),
	loc           = FuBar_TrackerLocals,
	hasIcon       = FuBar_TrackerLocals.DEFAULT_ICON,
	canHideText   = TRUE,
})
	
-- Methods
function FuBar_Tracker:IsShowingMiniMap()
	return self.data.showMiniMap
end
	
function FuBar_Tracker:ToggleShowMiniMap(loud)
	self.data.showMiniMap = not self.data.showMiniMap
	if loud then -- loud will be set to a string if called from the command line, thus being non-false.
		self.cmd:status(self.loc.ARGUMENT_MINIMAP, self.data.showMiniMap and 1 or 0, FuBarLocals.MAP_ONOFF)
	end
	if ( self.data.showMiniMap ) then
		MiniMapTrackingFrame:Show()
	else
		MiniMapTrackingFrame:Hide()
	end
	return self.data.showMiniMap
end

function FuBar_Tracker:Tracker_FindTracking(theTexture)
	--DEFAULT_CHAT_FRAME:AddMessage(theTexture)
	if ( theTexture == FuBar_TrackerLocals.TRACKER_ICON_FIND_HERBS ) then		
		return FuBar_TrackerLocals.TRACKER_SPELL_FIND_HERBS
	elseif ( theTexture == FuBar_TrackerLocals.TRACKER_ICON_FIND_MINERALS) then
		return FuBar_TrackerLocals.TRACKER_SPELL_FIND_MINERALS
	elseif ( theTexture == FuBar_TrackerLocals.TRACKER_ICON_FIND_TREASURE ) then
		return FuBar_TrackerLocals.TRACKER_SPELL_FIND_TREASURE
	elseif ( theTexture == FuBar_TrackerLocals.TRACKER_ICON_TRACK_BEASTS ) then
		return FuBar_TrackerLocals.TRACKER_SPELL_TRACK_BEASTS
	elseif ( theTexture == FuBar_TrackerLocals.TRACKER_ICON_TRACK_HUMANOIDS ) then
		return FuBar_TrackerLocals.TRACKER_SPELL_TRACK_HUMANOIDS
	elseif ( theTexture == FuBar_TrackerLocals.TRACKER_ICON_TRACK_HIDDEN ) then
		return FuBar_TrackerLocals.TRACKER_SPELL_TRACK_HIDDEN
	elseif ( theTexture == FuBar_TrackerLocals.TRACKER_ICON_TRACK_ELEMENTALS ) then
		return FuBar_TrackerLocals.TRACKER_SPELL_TRACK_ELEMENTALS
	elseif ( theTexture == FuBar_TrackerLocals.TRACKER_ICON_TRACK_UNDEAD ) then
		return FuBar_TrackerLocals.TRACKER_SPELL_TRACK_UNDEAD
	elseif ( theTexture == FuBar_TrackerLocals.TRACKER_ICON_TRACK_DEMONS ) then
		return FuBar_TrackerLocals.TRACKER_SPELL_TRACK_DEMONS
	elseif ( theTexture == FuBar_TrackerLocals.TRACKER_ICON_TRACK_GIANTS ) then
		return FuBar_TrackerLocals.TRACKER_SPELL_TRACK_GIANTS
	elseif ( theTexture == FuBar_TrackerLocals.TRACKER_ICON_TRACK_DRAGONKIN ) then 
		return FuBar_TrackerLocals.TRACKER_SPELL_TRACK_DRAGONKIN
	elseif ( theTexture == FuBar_TrackerLocals.TRACKER_ICON_SENSE_UNDEAD ) then
		return FuBar_TrackerLocals.TRACKER_SPELL_SENSE_UNDEAD
	elseif ( theTexture == FuBar_TrackerLocals.TRACKER_ICON_SENSE_DEMONS ) then
		return FuBar_TrackerLocals.TRACKER_SPELL_SENSE_DEMONS
	else
		return "No Match!"
	end
end
	
function FuBar_Tracker:Tracker_CastSpell()
	CastSpell(this.value, 0)
			
	if ( self.data.showMiniMap ) then
		MiniMapTrackingFrame:Show()
	else
		MiniMapTrackingFrame:Hide()
	end
end

function FuBar_Tracker:Tracker_CancelTrackingBuff()
	CancelTrackingBuff()
	--self:Update()
end

function FuBar_Tracker:IsTrackingChecked(val)
	return (self.data.currTexture == val)
end

function FuBar_Tracker:Initialize()
	self.data.currTexture = GetTrackingTexture()
	
	for _, spellName in ipairs(trackerSpells) do
			trackerLookUp[spellName] = 0;
	end
end
	
function FuBar_Tracker:Enable()
	self:RegisterEvent("SPELLS_CHANGED")
	self:RegisterEvent("LEARNED_SPELL_IN_TAB")
	self:RegisterEvent("PLAYER_AURAS_CHANGED")
end

function FuBar_Tracker:SPELLS_CHANGED()
	self:Update()
end

function FuBar_Tracker:LEARNED_SPELL_IN_TAB()
	self:Update()
end

function FuBar_Tracker:PLAYER_AURAS_CHANGED()
	self:Update()
end

function FuBar_Tracker:Disable()
	-- you do not need to unregister the event here, all events/hooks are unregistered on disable implicitly.
end
	
function FuBar_Tracker:Report()
	self.cmd:report({
		{text=self.loc.ARGUMENT_MINIMAP, val=(self:IsShowingMiniMap() and 1 or 0), map=FuBarLocals.MAP_ONOFF},
	})
end
	
function FuBar_Tracker:MenuSettings(level, value)
	if level == 1 then
		local menuLookUp = {}
		local numTrack = 0

		local i = 1
		local spellName, spellRank = GetSpellName(i, BOOKTYPE_SPELL)
		
		while spellName do
			if trackerLookUp[spellName] == 0 then
				menuLookUp[spellName] = i - 1
			end

			spellName, spellRank = GetSpellName(i, BOOKTYPE_SPELL)
			i = i + 1
		end

		for _, spellName in ipairs(trackerSpells) do
			if menuLookUp[spellName] ~= nil and menuLookUp[spellName] > 0 then
				numTrack = numTrack + 1
				
				dewdrop:AddLine(
					'text', spellName,
					--'value', menuLookUp[spellName],
					'arg1', menuLookUp[spellName],
					'func', function(val)
						CastSpell(val, 0)
				
						if ( self.data.showMiniMap ) then
							MiniMapTrackingFrame:Show()
						else
							MiniMapTrackingFrame:Hide()
						end
					end
					--'checked', self.data.currTexture == GetSpellTexture(menuLookUp[spellName],1) 
					--'checked', self:IsTrackingChecked(GetSpellTexture(menuLookUp[spellName],1))
				)
			end
		end
		
		if ( numTrack == 0 ) then
			dewdrop:AddLine(
				'text', self.loc.TRACKER_NOTHING_TEXT
			)
		else
			dewdrop:AddLine()
			dewdrop:AddLine(
				'text', self.loc.TRACKER_STOP_TRACKING,
				'func', function()
				--	CancelTrackingBuff()
					self:Tracker_CancelTrackingBuff()
				end
			)
		
			dewdrop:AddLine()
			
			dewdrop:AddLine(
				'text', self.loc.TRACKER_SHOW_MINIMAP,
				'func', function()
					self:ToggleShowMiniMap()
				end,
				'checked', self:IsShowingMiniMap()
			)
		end
	end
end
	
function FuBar_Tracker:UpdateData()
	if ( self.data.showMiniMap ) then
		MiniMapTrackingFrame:Show()
	else
		MiniMapTrackingFrame:Hide()
	end
	
	self.data.currTexture = GetTrackingTexture()
end
	
function FuBar_Tracker:UpdateText()
	if ( self.data.currTexture ~= nil ) then
		self:SetIcon(self.data.currTexture)
		self:SetText(self:Tracker_FindTracking(self.data.currTexture))
	else
		self:SetIcon(self.loc.DEFAULT_ICON)
		self:SetText(self.loc.TRACKER_OFF_TEXT)
	end
end
	
function FuBar_Tracker:UpdateTooltip()
		
	local tooltipRichText = "";
	
	if ( self.data.currTexture == nil) then
		tooltipRichText = self.loc.TRACKER_OFF_TEXT
	else
		tooltipRichText = self:Tracker_FindTracking(self.data.currTexture)
	end
	
	local cat = tablet:AddCategory()
	cat:AddLine(
		'text', tooltipRichText
	)
		
	tablet:SetHint(self.loc.TRACKER_TOOLTIP_HINT)
end
	
function FuBar_Tracker:OnClick()
	-- do something
end

FuBar_Tracker:RegisterForLoad()