--[[

	MiniGroup2Themes.lua

	This file contains the style layouts for MiniGroup2. 
	If you add a style to the MiniGroup2themes table in this file, it will automatically be added to MiniGroup2's list of frame styles.
	
	Wierd things will happen if the MiniGroup2themes table contains no or only broken styles. Don't do it.
	
	Each theme must contain Name, Tip, FrameHeight, FrameWidth and BarWidth, and of course ThemeData.
	Tags other than those are special case tags.
	ThemeData contains information about each element in the style. 
	
	Point, RelativePoint, x, y must be set for the element to be given a position.
	Height specifies the height of the element. 
	
	If you do not set a RelativeTo, the element will be relative to the unit frame.
	RelativeToSecondary is what the element will be relative to if the RelativeTo element is hidden.
	
	Things like icons should not have thier height or width set. 
	
	Visibility should contain an array of things that should be hidden too, if the given element is hidden.
	HeightAdd will expand the FrameHeight with the given value if the element is visible.
	Hidden = true/false will hide or show the element.
	
	Font and FontSize will set the style for a font string. Do not use these tags for elements that isn't a font.
	
	
	
	WARNING:
	--------------------------------------------------------------------------------------------------------------------------------------
	--		 	Wrong use of this table could cause MiniGroup2 not to work. Do not report bugs and errors in MiniGroup2					--
	--			unless you've made sure this file is untouched and original. I mean this! Just don't! :D								--
	--------------------------------------------------------------------------------------------------------------------------------------

	--------------------------------------------------------------------------------------------------------------------------------------
	--		 	MiniGroup2 is in development, so any themes you make may or may not work in a newer version of MiniGroup2				--
	--------------------------------------------------------------------------------------------------------------------------------------	
--]]
MiniGroup2themes = {
	ABF = {
		Name = "ABF",
		Tip = "ABF Tip",
		ResizeBars = false,
		BackgroundBarColor = true,
		AlphaBar = true,
		RaidColorName = false,
		PetClassName = true,	
		ComboGFX = true,
		HappinessBar = true,
		
		ThemeData = {
			all = {
				FrameHeight = 25,
				FrameWidth = 141,

				Combo1 = 	{ 		Point = "BOTTOMRIGHT",
									RelativePoint = "BOTTOMRIGHT",
									x = 1,
									y = -1,	
									Height = 10,
									HeightAdd = 0,									
									Width = 10,
								},
								
				HealthBar_BG = 	{ 	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT",
									x = 5,
									y = -5,	
									Visibility = {"HealthBar_BG","HealthBar","BarHealthText","HealthText",},
									Height = 15,
									HeightAdd = 0,									
									PetAdjust = -4,
									Width = 131,
								},
									
				ManaBar_BG = 	{	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT", 
									x = 5, 
									y = -21, 
									HeightAdd = 12,
									Visibility = {"ManaBar_BG","ManaBar","BarManaText","ManaText","ClassText",},	
									Height = 11,
									Width = 131,
								},
								
				XPBar_BG = 		{	Point = "TOPLEFT", 
									RelativeTo = "ManaBar_BG", 
									RelativeToSecondary = "HealthBar_BG", 
									RelativePoint = "BOTTOMLEFT", 
									x = 0, 
									y = -1, 
									HeightAdd = 5,
									Visibility = {"XPBar_BG","XPBar",},	
									Width = 131,
									Height = 4,
								},
				
				XPBar = 		{	Width = 131,
								},
								
				XPBar_Rest = 	{	Width = 131,
								},
									
				NameBackground=	{	Hidden = true,
								},
								
					Happiness =	{	Point = "TOPLEFT",
									RelativeTo = "HealthBar_BG",
									RelativePoint = "TOPLEFT", 
									x=0,
									y=0,
									Height = 15,
									Width = 131,
									Visibility = {"Happiness",},
								},
								
				NameLabel = 	{ 	Font = "Interface\\AddOns\\MiniGroup2\\fonts\\barframes.ttf", 
									FontSize = 9, 
									Width = 84, 
									Point = "LEFT", 
									RelativeTo = "HealthBar", 
									RelativePoint = "LEFT", 
									x = 2, 
									y =0,
								},
				
				ClassText =	 	{ 	Font = "Interface\\AddOns\\MiniGroup2\\fonts\\barframes.ttf",
									FontSize = 8, 
									Justify = "LEFT",
									Width = 60, 
									Point = "LEFT", 
									RelativeTo = "ManaBar", 
									RelativePoint = "LEFT", 
									x = 2, 
									y =0,
								},
					
				HealthText = 	{ 	Font = "Interface\\AddOns\\MiniGroup2\\fonts\\barframes.ttf", 
									FontSize = 9,
									Point = "RIGHT", 
									RelativeTo = "HealthBar_BG", 
									RelativePoint = "RIGHT", 
									x=-2,
									y=0,
								},
					
				ManaText = 		{ 	Font = "Interface\\AddOns\\MiniGroup2\\fonts\\barframes.ttf", 
									FontSize = 8,
									Point = "RIGHT", 
									RelativeTo = "ManaBar_BG", 
									RelativePoint = "RIGHT", 
									x=-2,
									y=0,
								},
									
				BarHealthText = { 	Font = "Interface\\AddOns\\MiniGroup2\\fonts\\barframes.ttf", 
									FontSize = 9,
								},
									
				BarManaText = 	{ 	Font = "Interface\\AddOns\\MiniGroup2\\fonts\\barframes.ttf", 
									FontSize = 8,
								},
				
				PVPIcon = 		{ 	Point = "TOPRIGHT", 
									RelativePoint = "TOPRIGHT", 
									x = 14, 
									y= 2,
								},
								
				LeaderIcon = 	{ 	Point = "TOPLEFT", 
									RelativePoint = "TOPLEFT", 
									x = -2, 
									y= 2,
								},
								
				MasterIcon = 	{ 	Point = "TOPLEFT", 
									RelativePoint = "TOPLEFT", 
									x = -5, 
									y= 2,
								},
			},
			raid = {
				FrameWidth = 110,
				HealthBar_BG = 	{ 	Width = 100,
								},
				ManaBar_BG = 	{	Width = 100,
								},
				XPBar_BG = 		{	Width = 100,
								},
				XPBar = 		{	Width = 100,
								},
				XPBar_Rest = 	{	Width = 100,
								},								
			},
		},
	},
	
	yaABF = {
		Name = "yaABF",
		Tip = "yet another abf style",
		ResizeBars = false,
		BackgroundBarColor = true,
		AlphaBar = true,
		RaidColorName = true,
		PetClassName = true,	
		ComboGFX = true,
		HappinessBar = true,
		
		ThemeData = {
			all = {
				FrameHeight = 25,
				FrameWidth = 141,

				Combo1 = 	{ 		Point = "BOTTOMRIGHT",
									RelativePoint = "BOTTOMRIGHT",
									x = 1,
									y = -1,	
									Height = 10,
									HeightAdd = 0,									
									Width = 10,
								},
								
				HealthBar_BG = 	{ 	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT",
									x = 5,
									y = -5,	
									Visibility = {"HealthBar_BG","HealthBar","BarHealthText","HealthText",},
									Texture = "Interface\\AddOns\\MiniGroup2\\Images\\Smooth.tga",
									Height = 15,
									HeightAdd = 0,									
									PetAdjust = -4,
									Width = 131,
								},
									
				ManaBar_BG = 	{	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT", 
									x = 5, 
									y = -21, 
									HeightAdd = 12,
									Visibility = {"ManaBar_BG","ManaBar","BarManaText","ManaText","ClassText",},	
									Height = 11,
									Width = 131,
								},
								
				XPBar_BG = 		{	Point = "TOPLEFT", 
									RelativeTo = "ManaBar_BG", 
									RelativeToSecondary = "HealthBar_BG", 
									RelativePoint = "BOTTOMLEFT", 
									x = 0, 
									y = -1, 
									HeightAdd = 5,
									Visibility = {"XPBar_BG","XPBar",},	
									Width = 131,
									Height = 4,
								},
				
				XPBar = 		{	Width = 131,
								},
								
				XPBar_Rest = 	{	Width = 131,
								},
									
				NameBackground=	{	Hidden = true,
								},
								
					Happiness =	{	Point = "TOPLEFT",
									RelativeTo = "HealthBar_BG",
									RelativePoint = "TOPLEFT", 
									x=0,
									y=0,
									Height = 15,
									Width = 131,
									Visibility = {"Happiness",},
								},
								
				NameLabel = 	{ 	Font = "Interface\\AddOns\\MiniGroup2\\fonts\\yabf.ttf", 
									FontSize = 9, 
									Width = 84, 
									Point = "LEFT", 
									RelativeTo = "HealthBar", 
									RelativePoint = "LEFT", 
									x = 2, 
									y =0,
								},
				
				ClassText =	 	{ 	Font = "Interface\\AddOns\\MiniGroup2\\fonts\\yabf.ttf",
									FontSize = 9, 
									Hidden = true,
								},
					
				HealthText = 	{ 	Font = "Interface\\AddOns\\MiniGroup2\\fonts\\yabf.ttf", 
									FontSize = 9,
									Point = "RIGHT", 
									RelativeTo = "HealthBar_BG", 
									RelativePoint = "RIGHT", 
									x=-2,
									y=0,
								},
					
				ManaText = 		{ 	Font = "Interface\\AddOns\\MiniGroup2\\fonts\\yabf.ttf", 
									FontSize = 9,
									Point = "RIGHT", 
									RelativeTo = "ManaBar_BG", 
									RelativePoint = "RIGHT", 
									x=-2,
									y=0,
								},
									
				BarHealthText = { 	Font = "Interface\\AddOns\\MiniGroup2\\fonts\\yabf.ttf", 
									FontSize = 9,
								},
									
				BarManaText = 	{ 	Font = "Interface\\AddOns\\MiniGroup2\\fonts\\yabf.ttf", 
									FontSize = 9,
								},
				
				PVPIcon = 		{ 	Point = "TOPRIGHT", 
									RelativePoint = "TOPRIGHT", 
									x = 14, 
									y= 2,
								},
								
				LeaderIcon = 	{ 	Point = "TOPLEFT", 
									RelativePoint = "TOPLEFT", 
									x = -2, 
									y= 2,
								},
								
				MasterIcon = 	{ 	Point = "TOPLEFT", 
									RelativePoint = "TOPLEFT", 
									x = -5, 
									y= 2,
								},
			},
			raid = {
				FrameWidth = 130,
				FrameHeight = 20,
				HealthBar_BG = 	{ 	Width = 121,
								},
				ManaBar_BG = 	{	Width = 121,
									Height = 5,
								},
				XPBar_BG = 		{	Width = 121,
									Hidden = true,
								},
				XPBar = 		{	Width = 121,
									Hidden = true,
								},
				XPBar_Rest = 	{	Width = 121,
									Hidden = true,
								},
				NameLabel = 	{ 	Font = "Interface\\AddOns\\MiniGroup2\\fonts\\yabf.ttf", 
									FontSize = 13,
								},
				HealthText = 	{ 	Font = "Interface\\AddOns\\MiniGroup2\\fonts\\yabf.ttf", 
									FontSize = 13,
								},
				ManaText =		{	Hidden = true,
								},
				BarHealthText =	{	Hidden = true,
								},
				BarManaText =	{	Hidden = true,
								},
			},
		},
	},
	
	Classic = {
		Name = "Classic",
		Tip = "Classic MG look",
		ResizeBars = true,
		BackgroundBarColor = false,
		AlphaBar = false,
		RaidColorName == false,
		ComboGFX = false,
		
		ThemeData = {
			all = {
				FrameHeight = 40,
				FrameWidth = 170,	

				Combo1 = 	{ 		Point = "BOTTOMRIGHT",
									RelativePoint = "BOTTOMRIGHT",
									x = 1,
									y = -1,	
									Height = 10,
									HeightAdd = 0,									
									Width = 10,
								},
				
				ComboText = 	{ 	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT",
									x = 9,
									y = -14,
									Font = "Fonts\\FRIZQT__.TTF",
									FontSize = 8,
									Justify = "LEFT",
									Width = 60, 
								},
													
				HealthBar_BG = 	{ 	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT",
									x = 9,
									y = -25,	
									Visibility = {"HealthBar_BG","HealthBar","BarHealthText","HealthText",},
									Height = 6,
									Width = 152,
								},
									
				ManaBar_BG = 	{	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT", 
									x = 9, 
									y = -33, 
									HeightAdd = 7,
									Visibility = {"ManaBar_BG","ManaBar","BarManaText","ManaText","ClassText",},		
									Height = 6,
									Width = 152,
								},
								
				XPBar_BG = 		{	Point = "TOPLEFT",  
									RelativePoint = "TOPLEFT", 
									x = 9, 
									y = -19, 
									Visibility = {"XPBar_BG","XPBar",},
									Width = 92,
									Height = 4,
								},
					XPBar = 	{	Width = 92,
								},
								
				XPBar_Rest = 	{	Width = 92,
								},
	
				NameBackground=	{	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT", 
									x=6,
									y=-2,
									Height = 18,
									Width = 158,
								},
					Happiness =	{	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT", 
									x=6,
									y=-2,
									Height = 18,
									Width = 158,
									Visibility = {"Happiness",},	
								},
								
				NameLabel = 	{ 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 9, 
									Width = 140, 
									Point = "TOPLEFT", 
									RelativePoint = "TOPLEFT", 
									x =8, 
									y =-5,
								},
					
				ClassText =	 	{ 	Font = "Fonts\\FRIZQT__.TTF",
									FontSize = 8,
									Justify = "RIGHT",
									Width = 60, 
									Point = "TOPRIGHT", 
									RelativePoint = "TOPRIGHT", 
									x =-7, 
									y =-14,
								},
				
				HealthText = 	{ 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 8,
									Point = "TOPRIGHT",
									RelativePoint = "TOPRIGHT",
									x = -7,
									y = -22,										
								},
				
				ManaText = 		{ 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 8,
									Point = "TOPRIGHT",
									RelativePoint = "TOPRIGHT",
									x = -7,
									y = -30,									
								},
								
				BarHealthText = { 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 8,
								},
									
				BarManaText = 	{ 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 8,
								},
									
				PVPIcon = 		{ 	Point = "TOPRIGHT", 
									RelativePoint = "TOPRIGHT", 
									x = 6, 
									y= -3,
								},
								
				LeaderIcon = 	{ 	Point = "RIGHT", 
									RelativeTo = "PVPIcon",
									RelativePoint = "LEFT", 
									x = 60, 
									y= -4,
								},
									
				MasterIcon = 	{ 	Point = "RIGHT", 
									RelativeTo = "LeaderIcon",
									RelativePoint = "LEFT", 
									x = 0, 
									y= 0,
								},
			},
		},
	},	
	
	Compact = {
		Name = "Compact",
		Tip = "Compact Tip",
		ResizeBars = true,
		BackgroundBarColor = false,
		AlphaBar = false,
		RaidColorName == false,
		ComboGFX = true,
		
		ThemeData = {
			all = {
				FrameHeight = 28,
				FrameWidth = 170,	

				Combo1 = 	{ 		Point = "BOTTOMRIGHT",
									RelativePoint = "BOTTOMRIGHT",
									x = 1,
									y = -1,	
									Height = 10,
									HeightAdd = 0,									
									Width = 10,
								},
								
				HealthBar_BG = 	{ 	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT",
									x = 9,
									y = -15,	
									Visibility = {"HealthBar_BG","HealthBar","BarHealthText","HealthText",},
									Height = 6,
									Width = 152,
								},
									
				ManaBar_BG = 	{	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT", 
									x = 9, 
									y = -23, 
									Visibility = {"ManaBar_BG","ManaBar","BarManaText","ManaText","ClassText",},		
									Height = 6,
									Width = 152,
									HeightAdd = 7,
								},
								
				XPBar_BG = 		{	Point = "TOPRIGHT",  
									RelativePoint = "TOPRIGHT", 
									x = -9, 
									y = -9, 
									Visibility = {"XPBar_BG","XPBar",},
									Width = 72,
									Height = 4,
									Hide = "ClassText",
								},
					XPBar = 	{	Width = 72,
								},
								
				XPBar_Rest = 	{	Width = 72,
								},
	
				NameBackground=	{	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT", 
									x=6,
									y=-2,
									Height = 15,
									Width = 158,
								},
					Happiness =	{	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT", 
									x=6,
									y=-2,
									Height = 15,
									Width = 158,
									Visibility = {"Happiness",},	
								},
								
				NameLabel = 	{ 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 9, 
									Width = 84, 
									Point = "TOPLEFT", 
									RelativePoint = "TOPLEFT", 
									x =7, 
									y =-4,
								},
					
				ClassText =	 	{ 	Font = "Fonts\\FRIZQT__.TTF",
									FontSize = 8,
									Justify = "RIGHT",
									Width = 60, 
									Point = "TOPRIGHT", 
									RelativePoint = "TOPRIGHT", 
									x =-7, 
									y =-4,
								},
				
				HealthText = 	{ 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 8,
									Point = "TOPRIGHT",
									RelativePoint = "TOPRIGHT",
									x = -7,
									y = -12,										
								},
				
				ManaText = 		{ 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 8,
									Point = "TOPRIGHT",
									RelativePoint = "TOPRIGHT",
									x = -7,
									y = -20,									
								},
								
				BarHealthText = { 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 8,
								},
									
				BarManaText = 	{ 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 8,
								},
									
				PVPIcon = 		{ 	Point = "TOPRIGHT", 
									RelativePoint = "TOPRIGHT", 
									x = 14, 
									y= 2,
								},
								
				LeaderIcon = 	{ 	Point = "TOPLEFT", 
									RelativePoint = "TOPLEFT", 
									x = -2, 
									y= 2,
								},
								
				MasterIcon = 	{ 	Point = "TOPLEFT", 
									RelativePoint = "TOPLEFT", 
									x = -5, 
									y= 2,
								},
			},
		},
	},
	
	Retro = {
		Name = "Retro",
		Tip = "The old look",
		ResizeBars = false,
		BackgroundBarColor = false,
		AlphaBar = false,
		RaidColorName == false,
		ComboGFX = true,
		
		ThemeData = {
			all = {
				FrameHeight = 42,
				FrameWidth = 170,	

				Combo1 = 	{ 		Point = "BOTTOMRIGHT",
									RelativePoint = "BOTTOMRIGHT",
									x = 1,
									y = -1,	
									Height = 10,
									HeightAdd = 0,									
									Width = 10,
								},
								
				HealthBar_BG = 	{ 	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT",
									x = 45,
									y = -25,	
									Visibility = {"HealthBar_BG","HealthBar","BarHealthText","HealthText",},
									Height = 8,
									Width = 115,
								},
									
				ManaBar_BG = 	{	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT", 
									x = 45, 
									y = -35, 
									Visibility = {"ManaBar_BG","ManaBar","BarManaText","ManaText","ClassText",},		
									Height = 4,
									Width = 115,
									HeightAdd = 7,	
								},
								
				XPBar_BG = 		{	Point = "TOPLEFT",  
									RelativePoint = "TOPLEFT", 
									x = 45, 
									y = -20, 
									Visibility = {"XPBar_BG","XPBar",},
									Width = 115,
									Height = 3,
								},
					XPBar = 	{	Width = 115,
								},
								
				XPBar_Rest = 	{	Width = 115,
								},
	
				NameBackground=	{	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT", 
									x=6,
									y=-2,
									Height = 18,
									Width = 158,
								},
					Happiness =	{	Point = "TOPLEFT",
									RelativePoint = "TOPLEFT", 
									x=6,
									y=-2,
									Height = 18,
									Width = 158,
									Visibility = {"Happiness",},	
								},
								
				NameLabel = 	{ 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 9, 
									Width = 80, 
									Point = "TOPLEFT", 
									RelativePoint = "TOPLEFT", 
									x =8, 
									y =-8,
								},
					
				ClassText =	 	{ 	Font = "Fonts\\FRIZQT__.TTF",
									FontSize = 9,
									Justify = "LEFT",
									Width = 60, 
									Point = "TOPLEFT", 
									RelativePoint = "TOPLEFT", 
									x =8, 
									y =-17,
									Hidden = true,
								},
				
				HealthText = 	{ 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 9,
									Point = "TOPRIGHT",
									RelativePoint = "TOPRIGHT",
									x = -8,
									y = -8,										
								},
				
				ManaText = 		{ 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 9,
									Point = "TOPRIGHT",
									RelativePoint = "TOPRIGHT",
									x = -7,
									y = -30,
									Hidden = true,
								},
								
				BarHealthText = { 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 8,
								},
									
				BarManaText = 	{ 	Font = "Fonts\\FRIZQT__.TTF", 
									FontSize = 8,
								},
									
				PVPIcon = 		{ 	Point = "TOPLEFT", 
									RelativePoint = "TOPLEFT", 
									x = 6, 
									y= -24,
								},
								
				LeaderIcon = 	{ 	Point = "TOPLEFT", 
									RelativePoint = "TOPLEFT", 
									x = 35, 
									y= -24,
								},
									
				MasterIcon = 	{ 	Point = "RIGHT", 
									RelativeTo = "LeaderIcon",				
									RelativePoint = "LEFT", 
									x = 0, 
									y= 0,
								},
			},
		},
	},	
}